//-----------------------------------------------------------------------------
//  Copyright (C) 2002-2007 Thomas S. Ullrich 
//
//  This file is part of "xyscan".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//  
//  Author: Thomas S. Ullrich
//  Last update: July 16, 2007
//-----------------------------------------------------------------------------
#ifndef xyscanStateMachine_h
#define xyscanStateMachine_h

class xyscanStateMachine {
    
public:
    xyscanStateMachine();
    
    bool allowKeyLeft();
    bool allowKeyRight();
    bool allowKeyUp();
    bool allowKeyDown();
    bool allowXYScan();
    bool doPrepareErrorXScan();
    bool doPrepareErrorYScan();
    bool allowScanXLeft();
    bool allowScanXRight();
    bool allowScanYLower();
    bool allowScanYUpper();
    bool removeErrorXScanSetup();
    bool removeErrorYScanSetup();
    bool enableSettingsMenu();
    bool disableSettingsMenu();
    bool allowWriteXY();
    bool allowWriteXYErrorX();
    bool allowWriteXYErrorY();
    bool allowWriteXYErrorXErrorY();
    
    void setScanErrorXSelected(bool);
    void setScanErrorYSelected(bool);
    void setErrorXScanPrepared(bool);
    void setErrorYScanPrepared(bool);
    void setXYScanDone(bool);
    void setScanXLeftDone(bool);
    void setScanXRightDone(bool);
    void setScanYLowerDone(bool);
    void setScanYUpperDone(bool);
    void setErrorXScanSetupRemoved(bool);
    void setErrorYScanSetupRemoved(bool);
    void setDataWritten(bool);
    
private:
    bool mScanErrorXSelected;
    bool mScanErrorYSelected;
    bool mErrorXScanPrepared;
    bool mErrorYScanPrepared;
    bool mXYScanDone;
    bool mScanXLeftDone;
    bool mScanXRightDone;
    bool mScanYLowerDone;
    bool mScanYUpperDone;
    bool mErrorXScanSetupRemoved;
    bool mErrorYScanSetupRemoved;
    bool mDataWritten;
    
private:
    void clear();
};

inline xyscanStateMachine::xyscanStateMachine() {clear();}

inline void xyscanStateMachine::clear()
{
    mScanErrorXSelected = false;
    mScanErrorYSelected = false;
    mErrorXScanPrepared = false;
    mErrorYScanPrepared = false;
    mXYScanDone = false;
    mScanXLeftDone = false;
    mScanXRightDone = false;
    mScanYLowerDone = false;
    mScanYUpperDone = false;
    mErrorXScanSetupRemoved = true;
    mErrorYScanSetupRemoved = true;
    mDataWritten = false;
}

inline bool xyscanStateMachine::allowKeyLeft()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (mXYScanDone)
            return false;
        else
            return true;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (mScanXLeftDone && mScanXRightDone)
            return false;
        else
            return true;
    }
    else
        return true;
}

inline bool xyscanStateMachine::allowKeyRight()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (mXYScanDone)
            return false;
        else
            return true;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (mScanXLeftDone && mScanXRightDone)
            return false;
        else
            return true;
    }
    else
        return true;
}

inline bool xyscanStateMachine::allowKeyUp()
{
    if (mScanErrorXSelected && !mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (mXYScanDone)
            return false;
        else
            return true;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (!mXYScanDone) return true;
        if (mScanXLeftDone && mScanXRightDone && mErrorXScanSetupRemoved)
            return true;
        else
            return false;
    }
    else
        return true;
}

inline bool xyscanStateMachine::allowKeyDown()
{
    if (mScanErrorXSelected && !mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (mXYScanDone)
            return false;
        else
            return true;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return true;
        if (!mXYScanDone) return true;
        if (mScanXLeftDone && mScanXRightDone && mErrorXScanSetupRemoved)
            return true;
        else
            return false;
    }
    return true;
}

inline bool xyscanStateMachine::allowXYScan()
{
    if (!mScanErrorXSelected && !mScanErrorYSelected)
        return true;
    else {
        if (mDataWritten) return true;
        if (mXYScanDone)
            return false;
        else
            return true;
    }
}

inline bool xyscanStateMachine::doPrepareErrorXScan()
{
    if (mScanErrorXSelected) {
        if (mDataWritten) return false;
        if (mXYScanDone && !mErrorXScanPrepared)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::doPrepareErrorYScan()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return false;
        if (mXYScanDone && !mErrorYScanPrepared)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mDataWritten) return false;
        if (!mErrorYScanPrepared && mScanXRightDone)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::allowScanXLeft()
{
    if (mScanErrorXSelected && !mScanErrorYSelected) {
        if (mErrorXScanPrepared && !mScanXLeftDone)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mErrorXScanPrepared && !mScanXLeftDone)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::allowScanXRight()
{
    if (mScanErrorXSelected && !mScanErrorYSelected) {
        if (mErrorXScanPrepared && mScanXLeftDone && !mScanXRightDone)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mErrorXScanPrepared && mScanXLeftDone && !mScanXRightDone)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::allowScanYLower()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mErrorYScanPrepared && !mScanYLowerDone)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mErrorXScanSetupRemoved && mErrorYScanPrepared && !mScanYLowerDone)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::allowScanYUpper()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mErrorYScanPrepared && mScanYLowerDone && !mScanYUpperDone)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (!mScanYUpperDone && mScanYLowerDone)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::removeErrorXScanSetup()
{
    if (mScanErrorXSelected && !mScanErrorYSelected) {
        if (mScanXRightDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mScanXRightDone && !mScanYLowerDone && !mErrorXScanSetupRemoved)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::removeErrorYScanSetup()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mScanYUpperDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mScanYUpperDone && !mErrorYScanSetupRemoved)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::enableSettingsMenu()
{
    if (mScanErrorYSelected) {
        if (mScanYUpperDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else {
        if (mScanErrorXSelected) {
            if (mScanXRightDone && !mDataWritten)
                return true;
            else
                return false;
        }
        else
            return false;
    }
}

inline bool xyscanStateMachine::disableSettingsMenu()
{
    if (mScanErrorXSelected) {
        if (mXYScanDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else {
        if (mScanErrorYSelected) {
            if (mXYScanDone && !mDataWritten)
                return true;
            else
                return false;
        }
        else
            return false;
    }
}

inline bool xyscanStateMachine::allowWriteXY()
{
    if (!mScanErrorXSelected && !mScanErrorYSelected)
        return true;
    else
        return false;
}

inline bool xyscanStateMachine::allowWriteXYErrorX()
{
    if (mScanErrorXSelected && !mScanErrorYSelected) {
        if (mScanXRightDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::allowWriteXYErrorY()
{
    if (!mScanErrorXSelected && mScanErrorYSelected) {
        if (mScanYUpperDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline bool xyscanStateMachine::allowWriteXYErrorXErrorY()
{
    if (mScanErrorXSelected && mScanErrorYSelected) {
        if (mScanYUpperDone && !mDataWritten)
            return true;
        else
            return false;
    }
    else
        return false;
}

inline void xyscanStateMachine::setScanErrorXSelected(bool val)     {mScanErrorXSelected = val;}
inline void xyscanStateMachine::setScanErrorYSelected(bool val)     {mScanErrorYSelected = val;}
inline void xyscanStateMachine::setErrorXScanPrepared(bool val)     {mErrorXScanPrepared = val; mErrorXScanSetupRemoved = false;}
inline void xyscanStateMachine::setErrorYScanPrepared(bool val)     {mErrorYScanPrepared = val; mErrorYScanSetupRemoved = false;}
inline void xyscanStateMachine::setXYScanDone(bool val)             {mXYScanDone = val;}
inline void xyscanStateMachine::setScanXLeftDone(bool val)          {mScanXLeftDone = val;}
inline void xyscanStateMachine::setScanXRightDone(bool val)         {mScanXRightDone = val;}
inline void xyscanStateMachine::setScanYLowerDone(bool val)         {mScanYLowerDone = val;}
inline void xyscanStateMachine::setScanYUpperDone(bool val)         {mScanYUpperDone = val;}
inline void xyscanStateMachine::setErrorXScanSetupRemoved(bool val) {mErrorXScanSetupRemoved = val;}
inline void xyscanStateMachine::setErrorYScanSetupRemoved(bool val) {mErrorYScanSetupRemoved = val;}
inline void xyscanStateMachine::setDataWritten(bool val)            {mDataWritten = val; if (mDataWritten) clear();}

#endif

